# Declare our package
package Games::AssaultCube;
use strict; use warnings;

# Initialize our version
use vars qw( $VERSION );
$VERSION = '0.02';

1;
__END__
=head1 NAME

Games::AssaultCube - The AssaultCube perl libraries

=head1 SYNOPSIS

	This is just a placeholder, please see the subclasses.

=head1 ABSTRACT

This module provides the supporting perl libraries around the popular game, AssaultCube.

=head1 DESCRIPTION

This module provides the supporting perl libraries around the popular game, AssaultCube. Please see the
subclasses for specific functionality.

The AssaultCube website can be found at L<http://assault.cubers.net/>

WARNING: The modules in this distribution have been tested against the latest AssaultCube version: 1.0.2 and
use at your own risk if you have an older/newer version :)

=head1 SUPPORT

You can find documentation for this module with the perldoc command.

    perldoc Games::AssaultCube

=head2 Websites

=over 4

=item * AnnoCPAN: Annotated CPAN documentation

L<http://annocpan.org/dist/Games-AssaultCube>

=item * CPAN Ratings

L<http://cpanratings.perl.org/d/Games-AssaultCube>

=item * RT: CPAN's request tracker

L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Games-AssaultCube>

=item * Search CPAN

L<http://search.cpan.org/dist/Games-AssaultCube>

=back

=head2 Bugs

Please report any bugs or feature requests to C<bug-games-assaultcube at rt.cpan.org>, or through
the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Games-AssaultCube>.  I will be
notified, and then you'll automatically be notified of progress on your bug as I make changes.

=head1 AUTHOR

Apocalypse E<lt>apocal@cpan.orgE<gt>

Props goes to Getty and the BS clan for the support!

This project is sponsored by L<http://cubestats.net>

=head1 COPYRIGHT AND LICENSE

Copyright 2009 by Apocalypse

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut
